'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isValidAttribute = isValidAttribute;
/**
 * Check if an attribute shoudl be rendered into the DOM.
 */

function isValidAttribute(value) {
  if (typeof value === 'boolean') return value;
  if (typeof value === 'function') return false;
  if (value === '') return true;
  if (value === undefined) return false;
  if (value === null) return false;
  if (value === false) return false;
  return true;
}

/**
 * Group an array of virtual elements by their key, using index as a fallback.
 */

var groupByKey = exports.groupByKey = function groupByKey(children) {
  return children.reduce(function (acc, child, i) {
    if (child != null && child !== false) {
      acc.push({
        key: String(child.key || i),
        item: child,
        index: i
      });
    }
    return acc;
  }, []);
};

/**
 * Is a vnode a thunk?
 */

var isThunk = exports.isThunk = function isThunk(node) {
  return node.type === '#thunk';
};

/**
 * Is a vnode a text node?
 */

var isText = exports.isText = function isText(node) {
  return node.type === '#text';
};

/**
 * Determine if two virtual nodes are the same type
 */

var isSameThunk = exports.isSameThunk = function isSameThunk(left, right) {
  return isThunk(left) && isThunk(right) && left.data.render === right.data.render;
};

/**
 * Create a node path, eg. (23,5,2,4) => '23.5.2.4'
 */

var createPath = exports.createPath = function createPath() {
  for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  return args.join('.');
};

/**
 * Find a child node at a given path. Takes any tree that uses a
 * 'children' key. This will work for both virtual nodes and real
 * DOM trees.
 */

var findNodeAtPath = exports.findNodeAtPath = function findNodeAtPath(path, tree) {
  var parts = path.split('.');
  var node = undefined;
  while (parts.length) {
    var index = parts.shift();
    node = tree.children[index];
  }
  return node;
};

/**
 * Slightly nicer insertBefore
 */

var insertAtIndex = exports.insertAtIndex = function insertAtIndex(parent, index, el) {
  var target = parent.childNodes[index];
  if (target) {
    parent.insertBefore(el, target);
  } else {
    parent.appendChild(el);
  }
};

/**
 * Remove an element at an index
 */

var removeAtIndex = exports.removeAtIndex = function removeAtIndex(DOMElement, index) {
  DOMElement.removeChild(DOMElement.childNodes[index]);
};